@@ -4,17 +4,15 @@ package POE::Loop::EV;
 # $Id: EV.pm 27 2008-01-29 19:42:57Z andyg $
 
 use strict;
+use POE::Loop::PerlSignals;
 
-our $VERSION = '0.06';
+our $VERSION = '0.10';
 
 # Everything plugs into POE::Kernel.
 package # hide me from PAUSE
     POE::Kernel;
 
-use strict;
-
 use EV;
-use POE::Kernel;
 
 # Loop debugging
 sub EV_DEBUG () { $ENV{POE_EV_DEBUG} || 0 }
@@ -22,15 +20,11 @@ sub EV_DEBUG () { $ENV{POE_EV_DEBUG} || 0 }
 # Global EV timer object
 my $_watcher_timer;
 
-# Global SIGCHLD watcher
-my $_child_watcher;
-
-# Global list of EV signal objects
-my %signal_events;
-
 # Global list of EV filehandle objects, indexed by fd number
 my @fileno_watcher;
 
+my $_async_watcher;
+
 my $DIE_MESSAGE;
 
 ############################################################################
@@ -54,33 +48,29 @@ sub loop_initialize {
     }
 
     # Set up the global timer object
-    $_watcher_timer = EV::periodic_ns( 0, 0, 0, \&_loop_timer_callback );
+    $_watcher_timer = EV::periodic( 0, 0, 0, \&_loop_timer_callback );
     
-    # Set up the callback for SIGCHLD
-    $_child_watcher = EV::child( 0, 0, \&_child_callback );
+    # Workaround so perl signals are handled
+    $_async_watcher = EV::check(sub { });
     
     $EV::DIED = \&_die_handler;
 }
 
 # Timer callback to dispatch events.
-my $last_time = time();
 sub _loop_timer_callback {
     my $self = $poe_kernel;
     
     EV_DEBUG && warn "_loop_timer_callback, at " . time() . "\n";
 
-    if (TRACE_STATISTICS) {
-        $self->_data_stat_add('idle_seconds', time() - $last_time);
-    }
-
     $self->_data_ev_dispatch_due();
     $self->_test_if_kernel_is_idle();
 
     # Transferring control back to EV; this is idle time.
-    $last_time = time() if TRACE_STATISTICS;
 }
 
 sub loop_finalize {
+    my $self = shift;
+    
     EV_DEBUG && warn "loop_finalize\n";
     
     foreach my $fd ( 0 .. $#fileno_watcher ) {
@@ -94,7 +84,8 @@ sub loop_finalize {
         }
     }
     
-    loop_ignore_all_signals();
+    $self->loop_ignore_all_signals();
+    undef $_async_watcher;
 }
 
 sub loop_attach_uidestroy {
@@ -108,7 +99,7 @@ sub loop_do_timeslice {
 sub loop_run {
     EV_DEBUG && warn "loop_run\n";
     
-    EV::loop();
+    EV::run();
     
     if ( defined $DIE_MESSAGE ) {
         my $message = $DIE_MESSAGE;
@@ -123,7 +114,7 @@ sub loop_halt {
     $_watcher_timer->stop();
     undef $_watcher_timer;
     
-    EV::unloop();
+    EV::break();
 }
 
 sub _die_handler {
@@ -133,106 +124,9 @@ sub _die_handler {
     # to stop the loop and get the error later
     $DIE_MESSAGE = $@;
     
-    # This will cause the EV::loop call in loop_run to return,
+    # This will cause the EV::run call in loop_run to return,
     # and cause the process to die.
-    EV::unloop();
-}
-
-############################################################################
-# Signal Handling
-############################################################################
-
-sub loop_watch_signal {
-    my ( $self, $signame ) = @_;
-    
-    # Child process has stopped.
-    # XXX: libev always sets a SIGCHLD handler
-    if ( $signame eq 'CHLD' or $signame eq 'CLD' ) {
-        $_child_watcher->start();
-        
-        return;
-    }
-    
-    EV_DEBUG && warn "loop_watch_signal( $signame )\n";
-    
-    $signal_events{ $signame } ||= EV::signal(
-        $signame,
-        sub {
-            if ( TRACE_SIGNALS ) {
-                my $pipelike = $signame eq 'PIPE' ? 'PIPE-like' : 'generic';
-                POE::Kernel::_warn "<sg> Enqueuing $pipelike SIG$signame event";
-            }
-            
-            EV_DEBUG && warn "_loop_signal_callback( $signame )\n";
-            
-            $poe_kernel->_data_ev_enqueue(
-                $poe_kernel, $poe_kernel, EN_SIGNAL, ET_SIGNAL, [ $signame ],
-                __FILE__, __LINE__, undef, time()
-            );
-        },
-    );
-}
-
-sub loop_ignore_signal {
-    my ( $self, $signame ) = @_;
-
-    if ( $signame eq 'CHLD' or $signame eq 'CLD' ) {
-        if ( $_child_watcher ) {
-            $_child_watcher->stop();
-        }
-        
-        return;
-    }
-
-    if ( defined $signal_events{ $signame } ) {
-        $signal_events{ $signame }->stop();
-    }
-}
-
-sub loop_ignore_all_signals {
-    my $self = shift;
-
-    map { $_->stop() if defined $_ } values %signal_events;
-    
-    %signal_events = ();
-    
-    if ( $_child_watcher ) {
-        $_child_watcher->stop();
-    }
-}
-
-sub _child_callback {
-    my $w = shift;
-    
-    my $pid    = $w->rpid;
-    my $status = $w->rstatus;
-    
-    EV_DEBUG && warn "_child_callback( $pid, $status )\n";
-    
-    if ( TRACE_SIGNALS ) {
-        POE::Kernel::_warn("<sg> POE::Kernel detected SIGCHLD (pid=$pid; exit=$status)");
-    }
-    
-    # Check for explicit SIGCHLD watchers, and enqueue explicit
-    # events for them.
-    if ( exists $poe_kernel->[KR_PIDS]->{$pid} ) {
-        my @sessions_to_clear;
-        while ( my ($ses_key, $ses_rec) = each %{ $poe_kernel->[KR_PIDS]->{$pid} } ) {
-            $poe_kernel->_data_ev_enqueue(
-                $ses_rec->[0], $poe_kernel, $ses_rec->[1], ET_SIGCLD,
-                [ 'CHLD', $pid, $status ],
-                __FILE__, __LINE__, undef, time(),
-            );
-            push @sessions_to_clear, $ses_rec->[0];
-        }
-        $poe_kernel->_data_sig_pid_ignore($_, $pid) foreach @sessions_to_clear;
-    }
-    
-    $poe_kernel->_data_ev_enqueue(
-        $poe_kernel, $poe_kernel, EN_SIGNAL, ET_SIGNAL,
-        [ 'CHLD', $pid, $status ],
-        __FILE__, __LINE__, undef, time()
-    );
+    EV::break();
 }
 
 ############################################################################
@@ -391,6 +285,16 @@ L<POE>, L<POE::Loop>, L<EV>
 
 Andy Grundman <andy@hybridized.org>
 
+=head1 CONTRIBUTORS
+
+=over
+
+=item *
+
+Dan Book <dbook@cpan.org>
+
+=back
+
 =head1 THANKS
 
 Brandon Black, for his L<POE::Loop::Event_Lib> module.
@@ -400,4 +304,4 @@ Brandon Black, for his L<POE::Loop::Event_Lib> module.
 POE::Loop::EV is free software; you may redistribute it and/or modify it
 under the same terms as Perl itself.
 
-=cut
\ No newline at end of file
+=cut
